/******************************************************************************
 * C-BUNG 03
 * Fliekommaarithmetik und Modularisierung 
 *           
 * (c) 2003, by Jan Iversen und Andreas Schibilla
 *****************************************************************************/


#include "ueb03.h"
#include "dreieck.h"



/******************************************************************************
 * Gibt die Hilfe aus
 *           
 * Die Fkt. gibt den Fehlercode 1 zurck
 *****************************************************************************/
int ShowUsage(void)
{
  printf("usage: ueb03 [-h, -a \"Seitenlaenge\", -g \"Seitenlaenge\", -h \"Seitenlaenge\"]\n");
  printf("        -h: Hilfe\n");
  printf("        -a: Ankathete\n");
  printf("        -g: Gegenkathete\n");
  printf("        -y: Hypotenuse\n");
  return 1;
}


/******************************************************************************
 * berprft ob String "-a" oder "-g" oder "-y" ist
 * 
 * p              -  Zeiger auf Anfang des Strings
 *           
 * Die Fkt. gibt zurck, ob String einer der o. g. ist
 *****************************************************************************/
int checkparam1(char * p)
{
 return (strlen(p)==2) && (p[0]=='-') && ( (p[1]=='a') || (p[1]=='g') || (p[1]=='y')  );
}


/******************************************************************************
 * Konvertiert einen String in einen double Wert
 * 
 * p              -  Zeiger auf Anfang des Strings
 * wert           -  Zeiger auf die Var. fr das Ergebnis
 *           
 * Die Fkt. gibt zurck, ob das Konvertieren geklappt hat
 *****************************************************************************/
int checkparam2(char * p, double * wert)
{
 /* return (*wert = atof(p)) > 0.0; */
 return (sscanf(p, "%lf", wert) == 1) && (*wert > 0.0); 
}


/******************************************************************************
 * Ermittelt, welche Seite des Dreiecks angegeben wurde
 * 
 * p              -  Zeiger auf Anfang des Strings
 *           
 * Die Fkt. gibt die Seite als Zahl zurck
 *****************************************************************************/
int welcheseite(char *p)
{
 switch (p[1])
   {
      case 'a': return 0;
      case 'g': return 1;
      case 'y': return 2;
      default: return 0;
   }
}


/******************************************************************************
 * Rundet eine Gleitk.Zahl auf 2 Kommastellen
 * 
 * num            -  Zu rundende Zahl
 *           
 * Die Fkt. gibt Ergebnis zurck
 *****************************************************************************/
double RoundTwo(double num)
{
 double tmp = num;
 
 tmp *= 100;                                             /* berchne erstmal abgerundete Zahl */
 tmp = floor(tmp);
 tmp /= 100;
 
 if ( (num - tmp) >= .005)                               /* wenn Nachkommstellen gro genug */
   tmp += 0.01;                                          /* aufrunden */

 return tmp;
}


/******************************************************************************
 * Gibt die eingegebenen Seitenlngen aus
 * 
 * p              -  String mit Zeichen fr Seite
 * wert           -  Lnge der Seite
 *           
 *****************************************************************************/
void SeitenAusgabe(char *p, double wert)
{
 switch (p[1])
   {
      case 'a': printf("Ankathete: %.2f ",      RoundTwo(wert) );    break;
      case 'g': printf("Gegenkathete: %.2f ",   RoundTwo(wert) );    break;
      case 'y': printf("Hypotenuse: %.2f ",     RoundTwo(wert) );    break;
   }
}


/******************************************************************************
 * Gibt den Winkel aus
 * 
 * winkel         -  Winkel als double
 *           
 *****************************************************************************/
void WinkelAusgabe(double winkel)
{
 printf("-> Winkel: %.2f Grad\n", winkel);
}


/******************************************************************************
 * Hauptfkt. berechnet Winkel des Dreiecks
 *
 * -> argc     - Wieviele Paramter
 * -> argv     - Array mit den Kommandozeilenparamtern als Strings
 *           
 * Die Fkt. gibt Returncode an Betriebssystem zurck
 *****************************************************************************/
int main (int argc, char *argv[])
{
 double  params[2]   = {0.0};                            /* speichert die beiden Zahlenwerte */
 double  dreieck[3]  = {0.0};                            /* speichert Werte fr a, g, y */
 double  winkel;                                         /* speichert berechneten Winkel */


 if ( (argc != 5)                      ||                /* es mssen 4 Paramter sein (+ Pfad) */
      !strcmp(argv[1], argv[3])        ||                /* die beiden Seiten mssen unterschiedlich sein */
      !checkparam1(argv[1])            ||                /* 1. Param. gltige Seitenangabe? */
      !checkparam1(argv[3])            ||                /* 3. Param. gltige Seitenangabe? */
      !checkparam2(argv[2],&params[0]) ||                /* 2. Param. gltiger Zahlenwert? */
      !checkparam2(argv[4],&params[1])                   /* 4. Param. gltiger Zahlenwert? */
   )      
   return ShowUsage();


 /* speicher Zahlenwerte im "Dreieck-Array" an richtiger Pos ab */
 dreieck[welcheseite(argv[1])] = params[0];
 dreieck[welcheseite(argv[3])] = params[1];


 /* Ist Hypotenuse lnger als Kathete? */
 if ( !ISTNULL(dreieck[2]) && ( (dreieck[0]>dreieck[2]) || (dreieck[1]>dreieck[2]) ) )
   return ShowUsage();


 /* Winkel berechnen */
 if ( ISTNULL(dreieck[0]) )                              /* Gegenkathete und Hyptenuse angegeben */
      winkel = WinkelGegenHypo(dreieck[1], dreieck[2]);

 if ( ISTNULL(dreieck[1]) )                              /* Ankathete und Hypotenuse angegeben */
      winkel = WinkelAnHypo(dreieck[0], dreieck[2]);

 if ( ISTNULL(dreieck[2]) )                              /* Ankathete und Gegenkathete angegeben */
      winkel = WinkelAnGegen(dreieck[0], dreieck[1]);


 /* Ist berechneter Winkel gerundet grer als Null? */
 if ( (winkel = RoundTwo(winkel)) <= 0.0)                
   return ShowUsage();


 SeitenAusgabe(argv[1], params[0]);                      /* erste Seitenlnge ausgeben */
 SeitenAusgabe(argv[3], params[1]);                      /* zweite Seitenlnge ausgeben */
 WinkelAusgabe(winkel);                                  /* Winkel ausgeben */
 
 return 0;
}

